package com.redisson.controller;

import org.redisson.api.RLock;
import org.redisson.api.RReadWriteLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author ty
 * @create 2021-12-15 9:52
 */
@RestController
public class RedissonLockController {

    @Autowired
    private RedissonClient redissonClient;

    /**
     * 可重入锁测试
     *      是否阻塞: 结果 会，
     * @return
     */
    @GetMapping("/test-lock")
    public String testLock() {
        // 1.获取锁，只要锁的名字一样，获取到的锁就是同一把锁。
        RLock lock = redissonClient.getLock("reentrant-lock");
        // 2.加锁
        lock.lock();
        System.out.println("加锁成功，线程id： " + Thread.currentThread().getId());
        try {
            Thread.sleep(10000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        } finally {
            // 释放锁
            lock.unlock();
            System.out.println("Finally，释放锁成功。线程 ID：" + Thread.currentThread().getId());
        }

        return "lock ok";
    }

    /**
     * 读写锁
     *  读+读 相当于没加锁
     *  读锁 + 写锁：写锁需要等待读锁释放锁。
     *  写锁 + 写锁：互斥，需要等待对方的锁释放。
     * @return
     */
    @GetMapping("/read-write-lock")
    public String readWriteLock() {
        RReadWriteLock lock = redissonClient.getReadWriteLock("read-write-lock");
        // 读锁
        RLock readLock = lock.readLock();
        // 写锁
        RLock writeLock = lock.writeLock();
        return "";
    }


}
